#include <QKeyEvent>
//#include <iostream>
#include <cmath>
#include "FlyElectrons.h"

FlyElectrons::FlyElectrons(QWidget *parent) : QGLWidget(parent) {
	moveSpeed = 1.0;
	for (int i = 0; i < 2; i++) electrons += new Electron(4, 0.6);
	for (int i = 0; i < 3; i++) electrons += new Electron(2, 0.6 + ((2.1 - 0.6) / 2.0));
	for (int i = 0; i < 4; i++) electrons += new Electron(1, 2.1);
	electrons += new Electron(1, 2.1, 2);
	connect(&timer, SIGNAL(timeout()), this, SLOT(onTimeout()));
	timer.start(1);
}

FlyElectrons::~FlyElectrons() {
	for (Electron *e : electrons) delete e;
}

void FlyElectrons::initializeGL() {
	static const GLfloat light_ambient[]  = { 0.0f, 0.0f, 0.0f, 1.0f };
	static const GLfloat light_diffuse[]  = { 1.0f, 1.0f, 1.0f, 1.0f };
	static const GLfloat light_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
	static const GLfloat light_position[] = { 2.0f, 5.0f, 5.0f, 0.0f };
	static const GLfloat mat_ambient[]    = { 0.7f, 0.7f, 0.7f, 1.0f };
	static const GLfloat mat_diffuse[]    = { 0.8f, 0.8f, 0.8f, 1.0f };
	static const GLfloat mat_specular[]   = { 1.0f, 1.0f, 1.0f, 1.0f };
	static const GLfloat high_shininess[] = { 100.0f };

	glClearColor(0.0, 0.0, 0.0, 0.0);
	quad = gluNewQuadric();
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);
	glEnable(GL_NORMALIZE);
	glEnable(GL_COLOR_MATERIAL);

	glLightfv(GL_LIGHT0, GL_AMBIENT,  light_ambient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE,  light_diffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);

	glMaterialfv(GL_FRONT, GL_AMBIENT,   mat_ambient);
	glMaterialfv(GL_FRONT, GL_DIFFUSE,   mat_diffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR,  mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, high_shininess);
}

void FlyElectrons::resizeGL(int width, int height) {
    glViewport(0, 0, width, height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
	GLfloat x = GLfloat(width) / height;
	glFrustum(-x, x, -1.0, 1.0, 4.0, 15.0);
    glMatrixMode(GL_MODELVIEW);
}

void FlyElectrons::paintGL() {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();
	glTranslatef(0.0, 0.0, -10);

	// Вертикальні лінії:
	glColor3f(1.0, 1.0, 1.0);
	glBegin(GL_LINES);
	for (float x = -2.2; x <= 2.2; x += 0.1) {
		glVertex3f(x, -2.4, 0);
		glVertex3f(x, 2.4, 0);
	}
	glEnd();

	// Ядро:
	glColor3f(1.0, 0.0, 0.0);
	gluSphere(quad, 0.15, 100, 20);

	// Електрони:
	for (Electron* e : electrons) {
		glColor3f(0.0, 0.0, (e->orbitType == 1) ? 1.0 : 0.5);
		glLoadIdentity();
		glTranslatef(0.0, 0.0, -10);
		glRotatef(e->rotationX, 1.0, 0.0, 0.0);
		glRotatef(e->rotationY, 0.0, 1.0, 0.0);
		glRotatef(e->rotationZ, 0.0, 0.0, 1.0);
		if (e->orbitType == 1) glTranslatef(e->orbitRadius, 0.0, 0.0);
		else glTranslatef(e->x, e->y, 0.0);
		gluSphere(quad, 0.2, 100, 20);
	}
}

void FlyElectrons::onTimeout() { // Спрацювання таймеру.
	for (Electron* e : electrons) {
		if (e->orbitType == 1) { // Кругла орбіта.
			e->rotationZ += moveSpeed * e->speed;
			if (e->rotationZ > 360.0) e->rotationZ -= 360.0;
		} else { // Орбіта 8.

			float f = sin(e->p);
			e->x = f * e->orbitRadius;

			if ((e->p > M_PI / 2) &&
					(e->p < ((M_PI * 2) - (M_PI / 2))))
				e->y = sin(f * M_PI + M_PI);
			else e->y = sin(f * M_PI);

			e->p += moveSpeed * e->speed / 30.0;

			if (e->p >= (M_PI * 2)) e->p = 0.0;

		}
	}
	updateGL();
}

void FlyElectrons::keyPressEvent(QKeyEvent *event) {
	switch (event->key()) {
	case Qt::Key_Plus:
		if (moveSpeed < 50.0) moveSpeed += 0.1;
		break;
	case Qt::Key_Minus:
		moveSpeed -= 0.1;
		if (moveSpeed < 0.1) moveSpeed = 0.1;
		break;
	case Qt::Key_F: case Qt::Key_F11:
		if (isFullScreen()) showNormal(); else showFullScreen();
		break;
	case Qt::Key_Escape:
		if (isFullScreen()) showNormal();
		break;
	}
}

void FlyElectrons::mouseDoubleClickEvent(QMouseEvent *) {
	if (isFullScreen()) showNormal(); else showFullScreen();
}
