#include <QKeyEvent>
#include <cmath>
#include <iostream>
#include "FlyElectrons2.h"

FlyElectrons2::FlyElectrons2(QWidget *parent) : QGLWidget(parent) { // Конструктор.
	position = 0;

	Coordinate c; float P = 2 * M_PI, R = 100, F = 0, E = 0.5, K = 10.07568;
	for (int i = 0; i <= 72; i++) {
		c.r = R / 100;
		c.a = F * (180 / M_PI);
		moveTable += c;
		std::cout << i + 1 << ": " << R << " " << F << std::endl;
		if (F > P) F -= P;
		F += K / R;
		R = 100 / (1 - E * sin(F));
		if (F > P) F -= P;
	}

	connect(&timer, SIGNAL(timeout()), this, SLOT(onTimeout()));
	timer.start(100);
}

FlyElectrons2::~FlyElectrons2() {}

void FlyElectrons2::initializeGL() {
	static const GLfloat light_ambient[]  = { 0.0f, 0.0f, 0.0f, 1.0f };
	static const GLfloat light_diffuse[]  = { 1.0f, 1.0f, 1.0f, 1.0f };
	static const GLfloat light_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
	static const GLfloat light_position[] = { 2.0f, 5.0f, 5.0f, 0.0f };
	static const GLfloat mat_ambient[]    = { 0.7f, 0.7f, 0.7f, 1.0f };
	static const GLfloat mat_diffuse[]    = { 0.8f, 0.8f, 0.8f, 1.0f };
	static const GLfloat mat_specular[]   = { 1.0f, 1.0f, 1.0f, 1.0f };
	static const GLfloat high_shininess[] = { 100.0f };

	glClearColor(0.0, 0.0, 0.0, 0.0);
	quad = gluNewQuadric();
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);
	glEnable(GL_NORMALIZE);
	glEnable(GL_COLOR_MATERIAL);

	glLightfv(GL_LIGHT0, GL_AMBIENT,  light_ambient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE,  light_diffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);

	glMaterialfv(GL_FRONT, GL_AMBIENT,   mat_ambient);
	glMaterialfv(GL_FRONT, GL_DIFFUSE,   mat_diffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR,  mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, high_shininess);
}

void FlyElectrons2::resizeGL(int width, int height) {
    glViewport(0, 0, width, height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
	GLfloat x = GLfloat(width) / height;
	glFrustum(-x, x, -1.0, 1.0, 4.0, 15.0);
    glMatrixMode(GL_MODELVIEW);
}

void FlyElectrons2::paintGL() {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();
	glTranslatef(0.0, 0.0, -10);


	// Ядро:
	glColor3f(1.0, 0.0, 0.0);
	gluSphere(quad, 0.15, 100, 20);

	// Електрон:
	glColor3f(0.0, 0.0, 1.0);
	glLoadIdentity();
	glTranslatef(0.0, 0.0, -10);
	/*
	glRotatef(e->rotationX, 1.0, 0.0, 0.0);
	glRotatef(e->rotationY, 0.0, 1.0, 0.0);
	*/
	glRotatef(moveTable[position].a, 0.0, 0.0, 1.0);
	glTranslatef(moveTable[position].r, 0, 0.0);
	gluSphere(quad, 0.2, 100, 20);
}

void FlyElectrons2::onTimeout() { // Спрацювання таймеру.
	position++;
	if (position >= moveTable.count()) position = 0;
	updateGL();
}

void FlyElectrons2::keyPressEvent(QKeyEvent *event) {
	switch (event->key()) {
	case Qt::Key_Plus:
		if (moveSpeed < 50.0) moveSpeed += 0.1;
		break;
	case Qt::Key_Minus:
		moveSpeed -= 0.1;
		if (moveSpeed < 0.1) moveSpeed = 0.1;
		break;
	case Qt::Key_F: case Qt::Key_F11:
		if (isFullScreen()) showNormal(); else showFullScreen();
		break;
	case Qt::Key_Escape:
		if (isFullScreen()) showNormal();
		break;
	}
}

void FlyElectrons2::mouseDoubleClickEvent(QMouseEvent *) {
	if (isFullScreen()) showNormal(); else showFullScreen();
}
