#include <QKeyEvent>
#include <cmath>
#include "HiperCube.h"

HiperCube::HiperCube(QWidget *parent) : QGLWidget(parent) {
	setWindowTitle("HiperCube");
	connect(&timer, SIGNAL(timeout()), this, SLOT(onTimeout()));
	timer.start(1);
}

void HiperCube::initializeGL() {
	glClearColor(0.0, 0.0, 0.0, 0.0);
}

void HiperCube::resizeGL(int width, int height) {
	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	GLfloat x = GLfloat(width) / height;
	glFrustum(-x, x, -1.0, 1.0, 4.0, 15.0);
	glMatrixMode(GL_MODELVIEW);
}

void HiperCube::paintGL() {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();
	glTranslatef(0.0, 0.0, -10);
	glRotatef(20, 1.0, 0.0, 0.0);
	glRotatef(20, 0.0, 1.0, 0.0);
	glRotatef(rotationZ, 0.0, 0.0, 1.0);

	glBegin(GL_LINES);

	drawCube(cube1Size);
	drawCube(cube2Size);

	// Сполучення вершин двох кубів.
	float d1 = cube1Size / 2, d2 = cube2Size / 2;

	glVertex3f(-d1, d1, d1);
	glVertex3f(-d2, d2, d2);

	glVertex3f(d1, d1, d1);
	glVertex3f(d2, d2, d2);

	glVertex3f(d1, d1, -d1);
	glVertex3f(d2, d2, -d2);

	glVertex3f(-d1, d1, -d1);
	glVertex3f(-d2, d2, -d2);

//------------------------------------------

	glVertex3f(-d1, -d1, d1);
	glVertex3f(-d2, -d2, d2);

	glVertex3f(d1, -d1, d1);
	glVertex3f(d2, -d2, d2);

	glVertex3f(d1, -d1, -d1);
	glVertex3f(d2, -d2, -d2);

	glVertex3f(-d1, -d1, -d1);
	glVertex3f(-d2, -d2, -d2);

	glEnd();
}

void HiperCube::drawCube(float size) {
	float d = size / 2;

	glVertex3f(-d, d, d);
	glVertex3f(d, d, d);

	glVertex3f(d, d, d);
	glVertex3f(d, d, -d);

	glVertex3f(d, d, -d);
	glVertex3f(-d, d, -d);

	glVertex3f(-d, d, -d);
	glVertex3f(-d, d, d);

//----------------------------------------

	glVertex3f(-d, -d, d);
	glVertex3f(d, -d, d);

	glVertex3f(d, -d, d);
	glVertex3f(d, -d, -d);

	glVertex3f(d, -d, -d);
	glVertex3f(-d, -d, -d);

	glVertex3f(-d, -d, -d);
	glVertex3f(-d, -d, d);

//-----------------------------------------

	glVertex3f(-d, d, d);
	glVertex3f(-d, -d, d);

	glVertex3f(d, d, d);
	glVertex3f(d, -d, d);

	glVertex3f(d, d, -d);
	glVertex3f(d, -d, -d);

	glVertex3f(-d, d, -d);
	glVertex3f(-d, -d, -d);
}

void HiperCube::onTimeout() { // Спрацювання таймеру.
	rotationZ -= 0.2;
	if (rotationZ <= 0.0) rotationZ = 360.0;

	position += 0.01;
	if (position >= (M_PI * 2)) position = 0.0;
	cube1Size = sin(position) + 1.7;
	cube2Size = sin(position + M_PI) + 1.7;

	updateGL();
}

void HiperCube::keyPressEvent(QKeyEvent *event) {
	switch (event->key()) {
	case Qt::Key_F: case Qt::Key_F11:
		if (isFullScreen()) showNormal(); else showFullScreen();
		break;
	case Qt::Key_Escape:
		if (isFullScreen()) showNormal();
		break;
	}
}

void HiperCube::mouseDoubleClickEvent(QMouseEvent *) {
	if (isFullScreen()) showNormal(); else showFullScreen();
}
