#include <QApplication>
#include <QImage>
#include <QPainter>
#include <iostream>
#include <cmath>

// Ця програма створює зображення в оперативній пам'яті
// і зберігає його в файл /tmp/temporary-image.png. Для перегляду файлу /tmp/temporary-image.png
// запускається програма eom.
// Зібрати програму можна командою: qmake-qt5 && make

void paintOnImage(QImage& image) {
	image.fill(Qt::white); // Заповнюємо зображення білим кольором.

	// Для малюваня по зображенню створюємо екземпляр класу QPainter
	// і передаємо йому адресу зображення.
	QPainter painter(&image);

	painter.drawPoint(250, 110);
	painter.drawLine(50, 40, 200, 60);
	painter.drawRect(60, 80, 140, 90);
	painter.drawEllipse(60, 200, 140, 90);
	painter.drawText(860, 370, "Привіт.");

	painter.setPen(QColor(Qt::green)); // Вибираємо зелений колір контуру.
	painter.drawEllipse(80, 320, 190, 190);

	painter.setPen(QColor(Qt::black));

	// Вибираємо світло синій колір фігури.
	// В QColor можна задати значення червоного, зеленого, синього кольору.
	painter.setBrush(QBrush(QColor(210, 210, 255)));

	painter.drawRect(300, 390, 110, 110);

	QPoint points1[5] = {
		QPoint(430, 410),
		QPoint(500, 420),
		QPoint(600, 590),
		QPoint(530, 590),
		QPoint(460, 550)};
	painter.drawPolygon(points1, 5); // Багатокутник.

	QPoint points2[5] = {
		QPoint(700, 430),
		QPoint(600, 420),
		QPoint(700, 590),
		QPoint(830, 590),
		QPoint(800, 450)};
	painter.drawPolyline(points2, 5); // Ламана.

	// Задаємо чорний колір, товщину 3 лінії.
	painter.setPen(QPen(QBrush(QColor(Qt::black)), 3));

	painter.drawLine(450, 390, 950, 390);

	for (double x = 300; x < 900; x += 0.1) {
		double y = sin(x / 30) * 150 + 180;
		// image.setPixelColor(x, y, QColor(Qt::red));
		image.setPixel(x, y, 0xff0000);
	}
}

int main(int argc, char *argv[]) {
	QApplication app(argc, argv); // Екземпляр класу QApplication потрібен для роботи бібліотеки Qt.

	// Клас QImage призначений для зберігання зображення в оперативній пам'яті.
	// Створюємо екземпляр класу QImage, в конструктор передаємо ширину, висоту та формат зображення.
	QImage image(1000, 650, QImage::Format_RGB32);

	paintOnImage(image);

	// Зберігаємо зображення в файл /tmp/temporary-image.png.
	if (!image.save("/tmp/temporary-image.png")) {
		std::cerr << "Помилка збереження зображення.";
		return 1;
	}

	system("eom /tmp/temporary-image.png");
	return 0;
}
