#include <vector>
#include <algorithm>
#include <numeric>
#include <iostream>

void show(std::vector<int>& v) {
	// for_each виконує функцію для кожного елементу контейнеру.
	std::for_each(v.begin(), v.end(), [](int i){ std::cout << i << ' '; });
	std::cout << std::endl;
}

int main() {
	auto f = [](int a, int b){ return a + b; }; // Іменована лямбда функція.
	std::cout << "Результат від f(5, 4): " << f(5, 4) << std::endl;

	std::vector<int> v({1, 2, 3, 4, 5, 6, 7, 8, 9});
	std::cout << "Зміст масиву v: ";
	show(v);
	std::cout << "Кількість парних чисел в масиві v: "
	          << std::count_if(v.begin(), v.end(), [](int i){ return !(i % 2); })
	          << std::endl;

	int s = 0;
	std::for_each(v.begin(), v.end(), [&s](int i){ s += i; });
	std::cout << "Сума чисел в масиві v: " << s << std::endl;
	// Завдяки [&s] лямбда функція має доступ по посилані до змінної s.
	// Якщо потрібен доступ по значенню, & між квадтратними дужкаим не потрібен.
	// Правило [] забороняє доступ до всіх зовнішних автоматичних змінн.
	// [=] - доступ по значенню до всіх автоматичних змінн.
	// [&] - доступ по посилані до всіх автоматичних змінн.

	std::cout << "Чи є в масиві v число менше 10: "
	          << std::any_of(v.begin(), v.end(), [](int i){ return i < 10; }) << std::endl;
	std::cout << "Чи є в масиві v число більше 9: "
	          << std::any_of(v.begin(), v.end(), [](int i){ return i > 9; }) << std::endl;

	auto it = std::find(v.begin(), v.end(), 5);
	if (it != v.end()) // Якщо в масиві v знайдено число 5.
		std::cout << "Індекс числа 5 в масиві v: " << (it - v.begin()) << std::endl;

	it = std::find_if(v.begin(), v.end(), [](int i){ return !(i % 2); });
	if (it != v.end()) // Якщо в масиві v знайдено парне число.
		std::cout << "Перше парне число в масиві v: " << *it << std::endl;

	std::replace(v.begin(), v.end(), 5, 9); // Замінюємо 5 на 9.
	std::cout << "Зміст масиву v після заміни 5 на 9: ";
	show(v);

	// Замінюємо парні числа на 10.
	std::replace_if(v.begin(), v.end(), [](int i){ return !(i % 2); }, 10);
	std::cout << "Зміст масиву v після заміни парних чисел на 10: ";
	show(v);

	std::cout << "Кількість чисел 10 в масиві v: "
	          << std::count(v.begin(), v.end(), 10) << std::endl;

	std::fill(v.begin(), v.end(), 5); // Заповнюємо масив v числом 5.
	std::cout << "Зміст масиву v після заповненням числом 5: ";
	show(v);

	// Заповнюємо масив v числами 1 - 9.
	std::generate(v.begin(), v.end(), [](){ static int i = 1; return i++; } );
	std::cout << "Зміст масиву v після заповненням числами 1 - 9: ";
	show(v);

	// Розташовуємо елементи масиву у зворотньому порядку.
	std::reverse(v.begin(), v.end());
	std::cout << "Зміст масиву v після функції reverse: ";
	show(v);

	srand(unsigned(time(nullptr))); // Ініціюємо генератор випадкових чисел.
	std::random_shuffle(v.begin(), v.end()); // Перемішуємо єлементи масиву.
	std::cout << "Зміст масиву v після перемішення: ";
	show(v);

	std::cout << "Найменше число в масиві v: "
	          << *std::min_element(v.begin(), v.end()) << std::endl
	          << "Найбільше число в масиві v: "
	          << *std::max_element(v.begin(), v.end()) << std::endl;

	std::sort(v.begin(), v.end()); // Сортуємо масив.
	std::cout << "Зміст масиву v після сортування: ";
	show(v);

	int a[9] = {0}, b[9] = {0};
	std::copy(v.begin(), v.end(), a); // Копіюємо масив v в масив a.
	std::cout << "Зміст масиву a: ";
	std::for_each(a, a + 9, [](int i){ std::cout << i << ' '; });
	std::cout << std::endl;

	std::copy(a, a + 9, b); // Копіюємо масив a в масив b.
	std::cout << "Зміст масиву b: ";
	std::for_each(b, b + 9, [](int i){ std::cout << i << ' '; });
	std::cout << std::endl;

	std::fill(a, a + 9, 0); // Заповнюємо масив a числом 0.

	// Копіюємо парні числа з масиву v в масив a.
	std::copy_if(v.begin(), v.end(), a, [](int i){ return !(i % 2); });
	std::cout << "Зміст масиву a після копіювання парних чисел з v: ";
	std::for_each(a, a + 9, [](int i){ std::cout << i << ' '; });
	std::cout << std::endl;

	// Множимо кожен елемент масиву v на 10.
	std::transform(v.begin(), v.end(), v.begin(), [](int i){ return i * 10; });
	std::cout << "Зміст масиву v після перетворення: ";
	show(v);

	std::cout << "Сума чисел в масиві v: " << std::accumulate(v.begin(), v.end(), 0) << std::endl;
	return 0;
}
