#include <vector>
#include <algorithm>
#include <iostream>

void show(std::vector<int>& v) {
	std::for_each(v.begin(), v.end(), [](int i){ std::cout << i << ' '; });
	std::cout << std::endl;
}

void show(auto begin, auto end) {
	std::for_each(begin, end, [](int i){ std::cout << i << ' '; });
	std::cout << std::endl;
}

int main() {
	std::vector<int> a({1, 2, 3, 4, 5, 6, 7, 8, 9}), v(a);
	std::cout << "Зміст масиву v: ";
	show(v);

	// Функція rotate крутить елементи масиву вліво. Другий
	// аргумент вказує адресу елементу, який має стати на початок иасиву.
	std::rotate(v.begin(), v.begin() + 1, v.end());
	std::cout << "Зміст v після прокрутки вліво на 1 елемент: ";
	show(v);

	v = a;
	std::rotate(v.begin(), v.begin() + 3, v.end());
	std::cout << "Зміст v після прокрутки вліво на 3 елементи: ";
	show(v);

	v = a;
	std::rotate(v.begin(), v.begin() + 8, v.end());
	std::cout << "Зміст v після прокрутки вліво на 8 елементи: ";
	show(v);

	std::fill(v.begin(), v.end(), 0);
	std::fill_n(v.begin(), 3, 5); // Заповнюємо в масиві v 3 елементи числом 5.
	std::cout << "Зміст v після заповнення 3-х елементів числом 5: ";
	show(v);

	std::copy_n(a.begin(), 5, v.begin()); // Копіюємо 5 елементи з a в v.
	std::cout << "Зміст v після копіювання 5 елементів з a: ";
	show(v);

	v = a;
	std::replace(v.begin(), v.end(), 5, 9);
	std::cout << "Зміст v після копіювання значень з a та заміни 5 на 9: ";
	show(v);

	// Видаляємо елементи з значенням 9 з масиву v.
	auto newEnd = std::remove(v.begin(), v.end(), 9);
	// Функція remove повертає ітератор на новий кінець масиву.
	std::cout << "Зміст v після видалення елементів з значенням 9: ";
	show(v.begin(), newEnd);
	std::cout << "Кількість видалених елементів з масиву v: "
	          << (v.end() - newEnd) << std::endl;

	v = a;
	std::cout << "Зміст v після копіювання значень з a: ";
	show(v);

	// Видаляємо парні числа з масиву v.
	newEnd = std::remove_if(v.begin(), v.end(), [](int i){ return !(i % 2); });
	std::cout << "Зміст v після видалення парних чисел: ";
	show(v.begin(), newEnd);
	std::cout << "Кількість видалених елементів з масиву v: "
	          << (v.end() - newEnd) << std::endl
	          << "Кількість елементів в масиві v: "
	          << (newEnd - v.begin()) << std::endl;

	v = {1, 2, 3, 4, 5, 5, 5, 6, 9, 9};
	std::cout << "Зміст v для перевірки: ";
	show(v);

	// Функція unique видаляє послідовні дублікати з масиву.
	newEnd = std::unique(v.begin(), v.end());
	std::cout << "Зміст v після видалення дублікатів: ";
	show(v.begin(), newEnd);

	return 0;
}
