#include <iostream>
#include <sstream>
#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/json_parser.hpp>
using namespace boost::property_tree;
using std::cout;

int main() {
	std::stringstream ss("{\n\
		\"id\": \"0001\",\n\
		\"type\": \"donut\",\n\
		\"name\": \"Pie\",\n\
		\"price\": \"$1\",\n\
		\"image\":\n\
			 {\n\
				 \"url\": \"images/1.jpg\",\n\
				 \"width\": 200,\n\
				 \"height\": 200\n\
			 }\n}";
	ptree pt;
	// Також буває ini_parser та xml_parser. Замість потіку можна передати шлях до файлу.
	json_parser::read_json(ss, pt);

	if ( pt.count("name") ) cout << pt.get<std::string>("name") << "\n";
	cout << pt.get("producer", "default value") << "\n";
	cout << pt.get("price", "$0") << "\n";
	cout << pt.get<std::string>("image.url") << "\n";

	ptree pt2 = pt.get_child("image");
	cout << pt2.get<std::string>("url") << "\n";

	pt.clear();
	pt.put("p1", "a");
	pt.put("p2", "b");
	pt.put("o.p1", 9);
	pt.put("o.p2", 5);
	write_json(cout, pt);

	ss.str("{\"a\": [\"one\", \"two\", \"three\", \"four\", \"five\"]}"); // Масив json.
	pt.clear();
	read_json(ss, pt);
	cout << "\n"
	write_json(cout, pt);

	cout << "\n";
	for (const ptree::value_type &v : pt.get_child("a")) // Перебір елементів масиву.
		// The data function is used to access the data stored in a node.
		cout << v.second.data() << "\n";

	// Визначаємо масив об'єктів.
	ss.str("{\"a\": ["
		   "{\"p1\": 1, \"p2\": 2}, {\"p1\": 3, \"p2\": 4}, {\"p1\": 5, \"p2\": 6}"
		   "]}");

	pt.clear();
	read_json(ss, pt);
	cout << "\n";
	for (const ptree::value_type &v : pt.get_child("a"))// Перебір елементів масиву.
		cout << "p1: " << v.second.get<int>("p1")
			 << ", p2: " << v.second.get<int>("p2") << "\n";

	// Створення масиву json. Єлемент такого масиву має пусте імя властивості.
	pt.clear();
	ptree child;
	for (int i = 1; i < 10; i++) {
		ptree element;
		element.put("", i);
		child.push_back(std::make_pair("", element));
	}
	pt.add_child("a", child);

	cout << "\n";
	write_json(cout, pt);
	return 0;
}
