#include <iostream>
#include <boost/asio.hpp>
#include <boost/bind.hpp>

class Session : public std::enable_shared_from_this<Session> {
public:
	Session(boost::asio::io_service& ios) : socket(ios) {}

	boost::asio::ip::tcp::socket& get_socket() {
		return socket;
	}

	void prepare_read() {
		socket.async_read_some(
			boost::asio::buffer(data, max_length),
			boost::bind(
				&Session::handle_read,
				this,
				shared_from_this(),
				boost::asio::placeholders::error,
				boost::asio::placeholders::bytes_transferred));
	}

	void handle_read(std::shared_ptr<Session>& /* s */,
	                 const boost::system::error_code& err,
	                 size_t bytes_transferred) {
		if (!err) {
			std::cout << "Received " << bytes_transferred << " bytes.\n";
			if (bytes_transferred) std::cout << std::string(data, bytes_transferred);
			prepare_read();
		} else {
			std::cerr << "Error: " << err.message() << std::endl;
		}
	}

private:
	boost::asio::ip::tcp::socket socket;
	enum { max_length = 1024 };
	char data[max_length];
};

class Server {
public:

	Server(short port)
	: acceptor(ios, boost::asio::ip::tcp::endpoint(boost::asio::ip::tcp::v4(), port)) {
		prepare_accept();
		ios.run();
	}

	void prepare_accept() {
		std::shared_ptr<Session> session = std::make_shared<Session>(ios);
		acceptor.async_accept(
			session->get_socket(),
			boost::bind(
				&Server::handle_accept,
				this,
				session,
				boost::asio::placeholders::error));
	}

	void handle_accept(std::shared_ptr<Session> session,
	                   const boost::system::error_code& err) {
		if (!err) {
			session->prepare_read();
			prepare_accept();
		} else {
			std::cerr << "Error: " + err.message() << std::endl;
			session.reset();
		}
	}

private:
	boost::asio::io_service ios;
	boost::asio::ip::tcp::acceptor acceptor;
};

int main() {
	try {
		Server s(8080);
	} catch(std::exception& e) {
		std::cerr << e.what() << std::endl;
	}
	return 0;
}
