#include <iostream>
#include <typeinfo>

struct A { virtual void vf() {}; };
struct B : A { void f() { std::cout << "B::f була викликана.\n"; } };
struct C : A { void f() { std::cout << "C::f була викликана.\n"; } };
enum e { a, b, c, d }; // c - d мають значення 0 - 3.

int main() {
	int i = 9;
	const char s[] = "string";

	void *p; // Універсальний вказівник.
	p = &i;

//	int n = *p; // Універсальний вказівник не може бути безпосередньо розіменований.

	int* ip = static_cast<int*>(p); // Явно привести тип можна оператором static_cast.
	std::cout << "*ip : " << *ip << std::endl;

	double d = 5.9;
	int n = static_cast<int>(d);
	std::cout << "n : " << n << std::endl;

	long l;
//	l = &i; // Не можна перетворити неявно вказівку в ціле.
//	l = static_cast<long>(&i); // static_cast не допоможе. Буде помилка компіляції.

	// Привести вказівку в ціле можна оператором reinterpret_cast.
	l = reinterpret_cast<long>(&i);
	std::cout << "l : " << l << std::endl;

//	ip = l; // Не можна перетворювати неявно ціле в вказівку. static_cast теж не допоможе.

	// Привести ціле в вказівку можна оператором reinterpret_cast.
	ip = reinterpret_cast<int*>(l);
	std::cout << "*ip : " << *ip << std::endl;

	i = 3;

	// Можна неявно перетворити enum в int.
	// Але не можна неявно перетворити int в enum тому що
	// змінна int може мати недопустиме значення для даного enum.
//	e v = i;

	// Якщо змінна int має гарантовано правильне значення, можна використати static_cast.
	e v = static_cast<e>(i);
	std::cout << "v: " << v << std::endl;

/*
Синтаксис операцій явного перетворення типів має такий вигляд:
cast-name< type >( expression );
де cast-name – одне із ключових слів: static_cast, const_cast, dynamic_cast
або reinterpret_cast, а type – тип, до якого приводиться вираз expression.

Чотири види явного перетворення були введені для того,
щоб врахувати всі можливі форми приведення типів.
Так const_cast необхідний для перетворення константного типу в неконстантний
та рухомого (volatile) – в нерухомий.

Оператор reinterpret_cast працює з внутрішніми представленями об’єктів
(re-interpret – інша інтерпритація того ж внутрішнього представлення).

Оператор приведення () є застарілим і еквівалентий оператору reіnterpret_cast.

Оператор dynamic_cast застосовується для перетворення вказівника або посилання, що посилається на
об'єкт типу класу у вказівник на тип класу з тієї ж ієрархії.
Його також використовують для трансформації l-значення об'єкта типу класу в
посилання на тип класу з тієї ж ієрархії.
Приведення типів за допомогою оператора dynamic_cast,
на відміну від інших наявних у C++ способів, здійснюється під час виконання програми.
*/

//	char* sp = s; // Не можна неявно константне перетворити в неконстантне.
	char* sp = const_cast<char *>(s); // Але можна використати оператор const_cast.
	std::cout << "sp : " << sp << std::endl;

	B b;
	// Можна неявно перетворити вказівку похідного класу на вказівку базового класу.
	A* pa = &b;

	// Не можна неявно перетворити вказівку базового класу на вказівку похідного класу.
//	B* pb = pa;

	B* pb = dynamic_cast<B*>(pa); // Але можна використати оператор dynamic_cast.
	// Оператор  dynamic_cast допустимий лише для поліморфного класу, тобто для класу який має
	//хоча б одну віртуальну функцію. Для неполіморного класу можна використати static_cast.
	// Можна перевірити правильність приведення.
	if (pb) pb->f();
	else std::cout << "Вказівка pa не вказує на екземпляр класу B.\n";

	A& a_reference = b;
	//a_reference.f(); // Трапится помилка компіляції ‘struct A’ has no member named ‘f’.

	// Перевірити правильність приведення посилання можна оператором try.
	try {
		B& b_reference = dynamic_cast<B&>(a_reference);
		b_reference.f();
	} catch (const std::bad_cast& e) {
		std::cout << e.what() << std::endl;
		std::cout << "Посилання a_reference не вказує на екземпляр класу B.\n";
	}

	try {
		C& c_reference = dynamic_cast<C&>(a_reference);
		c_reference.f();
	} catch (const std::bad_cast& e) {
		std::cout << e.what() << std::endl;
		std::cout << "Посилання a_reference не вказує на екземпляр класу C.\n";
	}
	return 0;
}
