#include <iostream>
#include <list>

// list це двузвязний список.
// В list операція вставки і видалення елементу швидша ніж в vector.
// Доступ до елементу list повільніший ніж в vector.

void show(std::list<int>& l) {
	for (std::list<int>::iterator it = l.begin(); it != l.end(); it++)
		std::cout << *it << ' ';
	std::cout << std::endl;
}

int main() {
	std::list<int> l = {5, 9, 8, 4, 2, 3, 6, 7, 1};
	std::cout << "l size: " << l.size() << std::endl;
	std::cout << "Is l empty: " << l.empty() << std::endl;
	show(l);

	// Функція front повертає посилання на перший елемент.
	// Функція back повертає посилання на останій елемент.
	std::cout << "First element of l: " << l.front() << std::endl;
	std::cout << "Last element of l: " << l.back() << std::endl;

	l.pop_front(); // Видалили перший елемент.
	l.pop_back(); // Видалили останій елемент.
	show(l);

	l.push_front(9); // Добавили елемент на початок.
	l.push_back(6); // Добавили елемент в кінець.
	show(l);

	// list не має оператору доступу по індексу [].
	// Змінити позицію ітератору можна функцією advance.
	std::list<int>::iterator it = l.begin();
	std::cout << *it << std::endl;
	std::advance(it, 3);
	std::cout << *it << std::endl;

	l.insert(it, 9); // Вставили елемиент.
	show(l);

	it = l.begin();
	std::advance(it, 4);
	l.erase(it); // Видалили елемиент.
	show(l);

	l.unique(); // Видалили дублікати які ідуть підряд.
	show(l);

	l.sort(); // Відсортували список.
	show(l);

	l.remove(6); // Видалили елементи по значеню.
	show(l);

	l.clear(); // Очистили список.
	std::cout << "l size: " << l.size() << std::endl;

	l.resize(30); // Змінили розмір списку.
	std::cout << "l size: " << l.size() << std::endl;

	int i = 1;
	for (it = l.begin(); it != l.end(); it++) *it = i++;
	show(l);

	l.reverse(); // Поставили елементи у зворотньому порядку.
	show(l);

	l.remove_if([](int n){ return n < 10; }); // Видалили елементи по умові.
	show(l);

	l.assign(3, 10); // Наповнили список. Передали кількість та значення.
	show(l);

	l.assign({5, 6, 7, 8, 9}); // Наповнили список.
	show(l);

	int a[] = {1, 2, 3, 4, 5};
	l.assign(a, a + 5);  // Наповнили список по масиву.
	show(l);

	return 0;
}
