#include <set>
#include <algorithm>
#include <iostream>

// Функція contains перевіряє чи містить контейнер елемент.
// Функції contains немає в STL.
template<class InputIterator, class T>
bool contains(InputIterator first, InputIterator last, const T& val)
{ return (find(first, last, val) != last); }

void show(const std::set<int>& s) {
	for_each(s.begin(), s.end(), [](int i){ std::cout << i << ' '; });
	std::cout << std::endl;
}

int main() {
	// Множина, set, містить тільки унікальні елементи і сортується автоматично.
	// У set значення елементу є його ключом.
	std::set<int> s({9, 1, 2, 7, 4, 6, 8});
	std::cout << "Зміст множини s: ";
	show(s);

	s.insert(5);
	s.insert(3);
	std::cout << "Зміст множини s після вставки 5 та 3: ";
	show(s);

	s.insert(5);
	s.insert(5);
	std::cout << "Зміст множини s після повторних вставок 5: ";
	show(s);

	std::cout << "Чи пуста множина s: " << s.empty() << std::endl
		 << "Чи є число 5 в множині s: " << contains(s.begin(), s.end(), 5) << std::endl
		 << "Розмір множини s: " << s.size() << std::endl;

	std::set<int>::iterator it = s.find(5);
	if (it != s.end()) // Якщо число 5 знайдено.
		std::cout << "Індекс числа 5 в множині s: "
			 << distance(s.begin(), it) << std::endl;
	// Функція distance визначає кількість елементів між ітераторами.
	// Елемент кінцевого ітератора не враховується.

	s.erase(5); // Видаляємо число 5 з множини s.
	// У функцію erase можна також передати ітератор або діапазон ітераторів.
	std::cout << "Зміст множини s після видалення числа 5: ";
	show(s);
	return 0;
}
