#include <iostream>
#include <tuple>

auto add_multiply_divide(int a, int b) {
	// Функція make_tuple спрощує створення кортежу.
	return std::make_tuple(a + b, a * b, double(a) / double(b));
}

std::tuple<int, int> f() {
	return {1, -1}; // Таким способом можна створити кортеж в C++17.
}

int main() {
	// Кортеж це колекція фіксованого розміру елементи якої можуть мати будь який тип.
	// Кортеж корисний наприклад для функції яка повертає більше 1 значення.

	std::tuple<int, float, std::string> t(5, 3.6, "string");

	// Отримати n-й элемент кортежу t можна таким способом: get<n>(t);
	std::cout << std::get<1>(t) << std::endl;

	std::get<1>(t) = 5.4; // Допустимо змінювати элементи кортежу.
	std::cout << std::get<1>(t) << std::endl;

	int a;
	float b;
	std::string c;
	std::tie(a, b, c) = t; // Таким способом можна розпакувати кортеж в змінні.
	std::cout << a << " " << b << " " << c << std::endl;

	// Якщо з кортежу треба отримати не всі елементи, можна використати std::ignore.
	std::tie(std::ignore, b, c) = t;
	std::cout << b << " " << c << std::endl;

	auto t2 = add_multiply_divide(5, 4);
	auto [x, y, z] = t2; // Такий спосіб розпаковки з'явився в C++ 17.
	std::cout << "5 + 4 = " << x << std::endl
	          << "5 * 4 = " << y << std::endl
	          << "5 / 4 = " << z << std::endl;

	auto t3 = std::tuple_cat(t, t2); // Конкатенація кортежів.
	std::cout << std::get<0>(t3) << " " << std::get<1>(t3) << " " << std::get<2>(t3) << " "
	          << std::get<3>(t3) << " " << std::get<4>(t3) << " " << std::get<5>(t3) << std::endl;

	// pair це вид кортежу для зберігання двух елементів.
	auto p = std::make_pair(9, "string");
	std::cout << p.first << " " << p.second << std::endl;
	std::pair p2{6, true}; // Так можна в C++17.
	std::cout << std::get<0>(p2) << " " << std::get<1>(p2) << std::endl;

	return 0;
}
