#include <iostream>
#include <sstream>
#include <vector>
#include <algorithm>
#include <iterator>

int main() {
	const int n = 15;
	int a[n] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15};

	// При записі в ітератор вставки відбувається вставка елементу в контейнер.
	// Бувають такі ітератори вставки:
	// back_insert_iterator - вставка в кінець.
	// front_insert_iterator - вставка в початок.
	// insert_iterator - вставка в певне місце.

	std::vector<int> v;
	std::back_insert_iterator<std::vector<int>> insert_it(v);
	std::copy(&a[0], &a[n], insert_it);
	// a[n] це неіснуючий елемент масиву, якій знаходиться після останього елементу.
	// Читати або записувати в a[n] не можна.
	// Але дозволяється отримувати адресу a[n] наприклад
	// щоб передати в функцію ітератор кінця від звичайного масиву.

	*insert_it = 16; // В кінець v вставиться 16.
	std::cout << "Розмір v: " << v.size() << std::endl;

	std::cout << "Елементи v: ";

	// Ітератор вихідного потіку ostream_iterator дозволяє записати дані з розділювачем на потік.
	std::copy( v.begin(), v.end(), std::ostream_iterator<int>(std::cout, " ") ); // " " is separator.
	std::cout << std::endl;

	// Функції back_inserter, front_inserter, inserter дозволяють зручно створити ітератор вставки.

	v.clear();

	// Копіюємо непарні числа.
	std::copy_if( &a[0], &a[n], std::back_inserter(v), [](int i) { return i % 2; } );

	std::vector<int>::iterator it; // it це простий ітератор.
	std::cout << "Обхід простим ітератором по v: ";
	for (it = v.begin(); it != v.end(); it++) std::cout << *it << " ";
	std::cout << std::endl;

	std::vector<int>::reverse_iterator reverse_it; // reverse_it це реверсний ітератор.
	std::cout << "Обхід реверсним ітератором по v: ";
	for (reverse_it = v.rbegin(); reverse_it != v.rend(); reverse_it++)
		std::cout << *reverse_it << " ";
	std::cout << std::endl;

	std::stringstream ss("1 2 3 4 5 6 7 8 9");
	v.clear();

	// istream_iterator дозволяє вводити дані з потіку через ітератор.
	std::copy(
			std::istream_iterator<int>(ss),
			std::istream_iterator<int>(), // Мітка "кінець потіку".
			std::back_inserter(v) );

	std::cout << "Елементи v після вводу даних з ss: ";
	std::copy( v.begin(), v.end(), std::ostream_iterator<int>(std::cout, " ") );
	std::cout << std::endl;
	return 0;
}
