#include <iostream>

int sum(int* array, int arraySize) {
	int r = 0;
	for (int i = 0; i < arraySize; i++) r += array[i];
	return r;
}

// Функція add_multiply_divide додає, множить та ділить 2 числа.
void add_multiply_divide(int n1, int n2, int* r1, int* r2, double* r3) {
	*r1 = n1 + n2;
	*r2 = n1 * n2;
	*r3 = double(n1) / double(n2);
}

int main() {
	//          0   1   2   3   4   5   6   7   8
	int a[9] = {10, 20, 30, 40, 50, 60, 70, 80, 90};

	std::cout << "Значення a[0]: " << a[0] << std::endl;
	/*
	Імя масиву a само по собі є вказівкою на перший елемент масиву.
	Через імя a можна прочитати адресу першого елементу масиву.
	Адреса елементу масиву це адреса першого байту елементу.
	*/
	std::cout << "Адреса першого елементу масиву a: " << a << std::endl;

	int *p;
	p = a; // Адресу першого елементу масиву a можна зберегти в вказівку.
	std::cout << "Значення p: " << p << std::endl;
	/*
	Операція * це читання або запис значення по адресі і називається розіменування.
	До зміни якогось імені цей термін не має відношення,
	скоріше це розкриття імені в певному сенсі.
	*/
	std::cout << "Результат від *p: " << *p << std::endl;

	// Значення елементу масиву на якій вказує p можна змінити розіменуванням.
	*p = 46;

	std::cout << "Результат від *p після зміни: " << *p << std::endl;
	std::cout << "Значення a[0] після зміни: " << a[0] << std::endl;

	//--------------------------------------------------------------------------------

	p = p + 1; // Також можна p += 1 або p++.
	std::cout << "Значення p після збільшення на 1: " << p << std::endl;
	std::cout << "Результат від *p після збільшення p на 1: " << *p << std::endl;

	/*
	Головна властивість вказівки.
	Оператори + - діють над вказівкою інакше ніж над простим числом.
	Якщо добавити до адреси вказівки число x,
	то адреса вказівки збільшиться на (x * розмір типу вказівки) байтів.

	Таким чином тепер p буде вказувати на перший байт наступного елементу масиву.
	Іншими словами, p буде вказувати на наступний елемент масиву.

	В оперативній пам'яті елементи масиву йдуть підряд без проміжків.
	Оператор - діє над вказівкою схожим чином але в іншу сторону.
	*/

	p = p + 1; // Ще раз перемістили вказівку на наступний елемент масиву.
	std::cout << "Значення p після повторного збільшення на 1: " << p << std::endl;
	std::cout << "Результат від *p після повторного збільшення p на 1: " << *p
			  << std::endl;

	//--------------------------------------------------------------------------------
	/*
	Перебір елементів масиву через вказівку.
	Для цього потрібно створити додаткову вказівку на кінець масиву.
	Цій вказівці має бути присвоєна адреса елементу, якій знаходиться після
	останього елементу масиву. Отримати цю адресу можна якщо до адреси першого
	елементу додати кількість елементів масиву.
	*/
	int* end = a + 9;
	/*
	C++ це дозволяє.
	Головне не виконати розіменування над вказівкою end. Інакше, в кращому випадку,
	нічого не трапится, в гіршому, програма аварійно завершиться
	або буде незрозуміла поведінка програми.
	Після останього елементу масиву знаходиться чужа область оперативної пам'яті.
	*/

	// Тепер можна в циклі збільшувати адресу вказівки p поки не натрапимо на end.
	p = a; // Ставимо вказівку p на перший елемент масиву a.
	while (p != end) {
		std::cout << *p << " ";
		p++; // Змінюємо p на адресу наступного елементу масиву.
	}
	std::cout << std::endl;

	// Інший варіант.
	for (p = a; p != end; p++)
		std::cout << *p << " ";
	std::cout << std::endl;

	//--------------------------------------------------------------------------------
	/*
	Вказівка може бути як звичайний масив.

	Над вказівкою також можна використовувати оператор доступу по індексу.
	p[індекс] діє так само як *(p + ікдекс).
	*(p + ікдекс) означає що до адреси в p додається індекс,
	результат збергається в регістру процесору і розіменовується. Сама вказівка p
	при цьому не змінює значення.
	*/

	p = a; // Ставимо вказівку p на перший елемент масиву a.

	 // Значення p[0] таке саме як a[0], p[1] таке саме як a[1], p[2] таке саме як a[2].
	std::cout << "Значення p[0]: " << p[0] << std::endl;
	std::cout << "Значення p[1]: " << p[1] << std::endl;
	std::cout << "Значення p[2]: " << p[2] << std::endl;

	std::cout << "Елементи масиву p: ";
	for (int i = 0; i < 9; i++) std::cout << p[i] << " ";
	std::cout << std::endl;

	// Імя масиву від вказівки відрізняє лиш те що імя масиву це константа.
	// Адресу першого елемненту, на яку вказує імя масиву, не можна змінити.
	// Вираз a = a + 1 дасть помилку.

	//--------------------------------------------------------------------------------
	// Вказівкою можна зробити підмасив.
	p = a + 5; // Для прикладу присвоємо вказівці адресу елементу масиву з індексом 5.
	/*
	Замість p = a + 5 можна також зробити:
	p = &a[5];
	& це оператор взяття адреси елементу масиву або змінної.
	В іншому місці & може бути оператором побітового або.
	*/

	// Значення p[0] таке саме як a[5], p[1] таке саме як a[6] і т.д.
	std::cout << "Значення p[0]: " << p[0] << std::endl;
	std::cout << "Значення p[1]: " << p[1] << std::endl;

	std::cout << "Елементи підмасиву p: ";
	for (int i = 0; i < 4; i++) std::cout << p[i] << " ";
	std::cout << std::endl;

	//--------------------------------------------------------------------------------
	// Вказівкою можна поділити масив на частини.
	int *part1 = a, *part2 = a + 3, *part3 = a + 6;

	std::cout << "Елементи масиву part1: ";
	for (int i = 0; i < 3; i++) std::cout << part1[i] << " ";
	std::cout << std::endl;
	std::cout << "Елементи масиву part2: ";
	for (int i = 0; i < 3; i++) std::cout << part2[i] << " ";
	std::cout << std::endl;
	std::cout << "Елементи масиву part3: ";
	for (int i = 0; i < 3; i++) std::cout << part3[i] << " ";
	std::cout << std::endl;

	// Для ділення масиву також можна використати масив вказівок.
	int* parts[3] = {a, a + 3, a + 6};
	/*
	Також можна зробити так:
	int* parts[3];
	parts[0] = a;
	parts[1] = a + 3;
	parts[2] = a + 6;
	*/
	for (int i = 0; i < 3; i++) {
		std::cout << "Елементи масиву parts[" << i << "]: ";
		for (int n = 0; n < 3; n++) std::cout << parts[i][n] << " ";
		std::cout << std::endl;
	}

	//--------------------------------------------------------------------------------
	/*
	Передача масиву в функцію.
	В функцію масив передається через вказівку.
	Параметр функції, який приймає масив, роблять вказівкою. В функцію передається
	адреса першого елементу масиву. Також в функцію потрібно передати
	в окремий параметр кількість елементів масиву.
	*/
	std::cout << "Сума чисел масиву a: " << sum(a, 9) << std::endl;

	//--------------------------------------------------------------------------------
	// Вказівка може працювати з звичайною змінною.

	int d = 139;
	p = &d; // Присвоємо вказівці p адресу змінної d.
	std::cout << "Значення змінної d: " << d << std::endl;
	std::cout << "Значення p: " << p << std::endl;
	std::cout << "Результат від *p: " << *p << std::endl;
	*p = 963; // Змінили значення змінної d через вказівку p.
	std::cout << "Значення змінної d: " << d << std::endl;
	std::cout << "Результат від *p: " << *p << std::endl;

	// Така здатність вказівки використовується наприклад для передачі адрес
	// змінних функції для того щоб функція могла повернути кілька значень.
	int result1, result2;
	double result3;
	add_multiply_divide(3, 6, &result1, &result2, &result3);
	std::cout << "3 + 6 = " << result1 << std::endl;
	std::cout << "3 * 6 = " << result2 << std::endl;
	std::cout << "3 / 6 = " << result3 << std::endl;

	return 0;
}
