#include <iostream>
#include <chrono>
#include <thread>
//#include <mutex>

uint64_t createRandomArray(int* array, int arraySize, int maxValue) {
	uint64_t checksum = 0;
	srand(unsigned(time(nullptr)));
	for (int i = 0; i < arraySize; i++) {
		array[i] = rand() % (maxValue + 1);
		checksum += array[i];
	}
	return checksum;
}

uint64_t getArrayChecksum(int* array, int arraySize) {
	uint64_t checksum = 0;
	for (int i = 0; i < arraySize; i++) checksum += array[i];
	return checksum;
}

void copyArray(int* array1, int* array2, int arraySize) {
	for (int i = 0; i < arraySize; i++) array2[i] = array1[i];
}

bool isArraySorted(int* array, int arraySize) {
	for (int i = 0; i < (arraySize - 1); i++) {
		if (array[i] > array[i + 1]) return false;
	}
	return true;
}

void showArray(int* array, int arraySize) {
	for (int i = 0; i < arraySize; i++) std::cout << array[i] << ' ';
	std::cout << std::endl;
}

void sortArrayMethod1(int* array, int arraySize) { // Метод дурня.
	int i = 0;
	while (i < (arraySize - 1)) {
		if (array[i] > array[i + 1]) {
			int t = array[i];
			array[i] = array[i + 1];
			array[i + 1] = t;
			i = 0;
		} else {
			i++;
		}
	}
}

void sortArrayMethod2(int* array, int arraySize) { // Проста бульбашка.
	bool b;
	do {
		b = false;
		for(int i = 0; i < (arraySize - 1); i++) {
			if (array[i] > array[i + 1]) {
				int t = array[i];
				array[i] = array[i + 1];
				array[i + 1] = t;
				b = true;
			}
		}
	} while(b);
}

void sortArrayMethod3(int* array, int arraySize) { // Просунута бульбашка.
	int i = 0;
	while (i < (arraySize - 1)) {
		if (array[i] > array[i + 1]) {
			int t = array[i];
			array[i] = array[i + 1];
			array[i + 1] = t;
			if (i) i--;
		} else {
			i++;
		}
	}
}

void sortArrayMethod4(int* array, int arraySize) { // Метод сортування вибіркою.
	for (int i = 0; i < (arraySize - 1); i++) {

		int min = i;
		for (int y = i + 1; y < arraySize; y++) {
			if (array[y] < array[min]) min = y;
		}

		if (min != i) {
			int t = array[i];
			array[i] = array[min];
			array[min] = t;
		}
	}
}

void radixSort(int* array, int arraySize, int maxValue) { // Розрядне сортування.
	int* m0 = new int[arraySize];
	int* m1 = new int[arraySize];

	for(int radix = 0; radix < 32; radix++) {
		int m0Size = 0, m1Size = 0, checkBit = 1 << radix, p = 0;
		if (checkBit > maxValue) break;

		for (int i = 0; i < arraySize; i++) {
			if (array[i] & checkBit)
				m1[m1Size++] = array[i];
			else
				m0[m0Size++] = array[i];
		}

		for (int i = 0; i < m0Size; i++) array[p++] = m0[i];
		for (int i = 0; i < m1Size; i++) array[p++] = m1[i];
	}

	delete[] m0;
	delete[] m1;
}

void countSort(int* array, int arraySize, int maxValue) { // Сортування підрахунком.
	int* m = new int[maxValue + 1];
	for (int i = 0; i < (maxValue + 1); i++) m[i] = 0;
	for (int i = 0; i < arraySize; i++) m[array[i]]++;
	int p = 0;
	for (int i = 0; i < (maxValue + 1); i++) {
		for (int k = 0; k < m[i]; k++) array[p++] = i;
	}
	delete[] m;
}

//std::mutex mtx;

// Сортування злиттям ділить масив на дві половини. Над половинами масиву рекурсивно викликається
// сортуюча функція. Після цього виконується сортуюче злиття половин масиву.
void mergeSort(int* array, int arraySize) { // Рекурсивне сортування злиттям.
/*
	mtx.lock();
	std::cout << "megreSort was invoked with array size " << arraySize << ".\n";
 	mtx.unlock();
 */
 	if (arraySize <= 1) return;

	int middle = arraySize / 2;

	int* left = array;
	int* right = array + middle;

	int leftSize = middle;
	int rightSize = arraySize - leftSize;

	mergeSort(left, leftSize);
	mergeSort(right, rightSize);

	int* buffer = new int[arraySize];
	int left_i = 0, right_i = 0, buffer_i = 0;

	while (left_i < leftSize and right_i < rightSize) {
		if (left[left_i] <= right[right_i]) {
			buffer[buffer_i] = left[left_i];
			left_i++;
		} else {
			buffer[buffer_i] = right[right_i];
			right_i++;
		}
		buffer_i++;
	}

	while (left_i < leftSize) {
		buffer[buffer_i] = left[left_i];
		left_i++;
		buffer_i++;
	}

	while (right_i < rightSize) {
		buffer[buffer_i] = right[right_i];
		right_i++;
		buffer_i++;
	}

	for (int i = 0; i < arraySize; i++) array[i] = buffer[i];
	delete[] buffer;
}

void multithreadMergeSort(int* array, int arraySize) { // Мультипоточне рекурсивне сортування злиттям.
	int processorCoresCount = std::thread::hardware_concurrency(); // Визначаємо число ядер процесору.
//	std::cout << "Processor cores count: " << processorCoresCount << std::endl;

	int partsCount = processorCoresCount; // partsCount це кількість шматків.
//	std::cout << "Parts count: " << partsCount << std::endl;

	int partSize = arraySize / partsCount; // Визначаємо мінімальний розмір шматка мисиву.
//	std::cout << "Minimum part size: " << partSize << std::endl;

	int* parts[partsCount]; // Шматки масиву.
	int partsSizes[partsCount]; // Розміри шматків.

	// Розкладаємо масив на шматки.
	int* p = array; // p спочатку вказує на адресу першого епементу масиву.
	for (int i = 0; i < partsCount; i++) {
		parts[i] = p;
		if (i == partsCount - 1) { // Якщо це останій шматок.
			// Розмір останього шматка може бути більший ніж partSize. 
			// Розмір останього шматка буде partSize + (остача ділення arraySize на partsCount).
			partsSizes[i] = partSize + (arraySize % partsCount);
		} else { // Якщо це не останій шматок.
			partsSizes[i] = partSize; // Розмір цього шматка буде partSize;
			p += partSize;  // Переміщаємо вказівку p на partSize елементів масиву.
		}
	}

//	for (int i = 0; i < partsCount; i++)
//		std::cout << "Part "<< i << " size: " << partsSizes[i] << std::endl;

	// Запускаємо потоки.
	std::thread* threads[partsCount];
	for (int i = 0; i < partsCount; i++)
		threads[i] = new std::thread(mergeSort, parts[i], partsSizes[i]);

	for (int i = 0; i < partsCount; i++) threads[i]->join(); // Чекаємо завершення потоків.

	//--------------------------------------------------------------------------------
	// Зливаємо шматки в один масив.

	// Ідея злиття одночасно багатьох шматків в масив в тому щоб по черзі
	// вибирати той шматок, в якого перше число менше ніж перше число інших шматків
	// і переміщати те число в буфер. Після цього скопіювати буфер в масив.

	int currentIndexes[partsCount]; // Поточні індекси для кожного шматка.
	// Тут слово "поточні" не відноситься до слова потік.
	for (int i = 0; i < partsCount; i++) currentIndexes[i] = 0;

 	int* buffer = new int[arraySize];
	int* part;
	int partCurrentIndex;
	int minPartIndex;
	int* minPart;
	int minPartCurrentIndex;

	for (int buffer_i = 0; buffer_i < arraySize; buffer_i++) {
		minPartIndex = -1;
		minPart = nullptr;

		// Знаходимо в якого шматка поточний індекс вказує на найменше число.
		for (int i = 0; i < partsCount; i++) { // Перебираємо шматки масиву.
			part = parts[i];

			// Якщо поточний індекс шматка вийшов за границю, пропускаємо шматок.
			if (currentIndexes[i] >= partsSizes[i]) continue;

			if (minPartIndex == -1) { // Якщо minPartIndex не ініційовано.

				 // Вибираємо шматок.
				minPartIndex = i;
				minPart = part;

				continue; // Йдемо на наступний шматок.
			}

			partCurrentIndex = currentIndexes[i];
			minPartCurrentIndex = currentIndexes[minPartIndex];
			// Якщо поточний індекс шматка вказує на менше число ніж в того що було знайдено раніше:
			if (part[partCurrentIndex] < minPart[minPartCurrentIndex]) {
				// Вибираємо шматок.
				minPartIndex = i;
				minPart = part;
			}
		} // for (int i = 0; i < partsCount; i++)

		if (minPartIndex == -1 or minPart == nullptr) { // Такої ситуації не повинно бути.
			std::cerr << "The multithreadMergeSort algorithm is wrong.\n";
			abort();
			return;
		}

		minPartCurrentIndex = currentIndexes[minPartIndex];
		// Копіюємо поточний елемент вибраного шматка в буфер.
		buffer[buffer_i] = minPart[minPartCurrentIndex];

		currentIndexes[minPartIndex]++; // Збільшуємо поточний індекс вибраного шматка на 1.
	} // for (int buffer_i = 0; buffer_i < arraySize; buffer_i++)

	for (int i = 0; i < arraySize; i++) array[i] = buffer[i];
	delete[] buffer;
	for (int i = 0; i < partsCount; i++) delete threads[i];
}

int main() {
	const int n = 10'000'000;
	const int maxValue = 1000;

	int* randomArray = new int[n];
	int* array = new int[n];
	std::cout << "array size: " << n << std::endl;
	std::chrono::high_resolution_clock::time_point t1, t2;
	int duration;

	uint64_t checksum = createRandomArray(randomArray, n, maxValue);
	std::cout << "array checksum: " << checksum << std::endl;
/*
	copyArray(randomArray, array, n);
	t1 = std::chrono::high_resolution_clock::now();
	sortArrayMethod1(array, n);
	t2 = std::chrono::high_resolution_clock::now();
	auto duration = std::chrono::duration_cast<std::chrono::milliseconds>(t2 - t1).count();
	std::cout << "sortArrayMethod1 duration: " << duration << " ms.\n";

	copyArray(randomArray, array, n);
	t1 = std::chrono::high_resolution_clock::now();
	sortArrayMethod2(array, n);
	t2 = std::chrono::high_resolution_clock::now();
	duration = std::chrono::duration_cast<std::chrono::milliseconds>(t2 - t1).count();
	std::cout << "sortArrayMethod2 duration: " << duration << " ms.\n";

	copyArray(randomArray, array, n);
	t1 = std::chrono::high_resolution_clock::now();
	sortArrayMethod3(array, n);
	t2 = std::chrono::high_resolution_clock::now();
	duration = std::chrono::duration_cast<std::chrono::milliseconds>(t2 - t1).count();
	std::cout << "sortArrayMethod3 duration: " << duration << " ms.\n";

	copyArray(randomArray, array, n);
	t1 = std::chrono::high_resolution_clock::now();
	sortArrayMethod4(array, n);
	t2 = std::chrono::high_resolution_clock::now();
	duration = std::chrono::duration_cast<std::chrono::milliseconds>(t2 - t1).count();
	std::cout << "sortArrayMethod4 duration: " << duration << " ms.\n";
*/
	copyArray(randomArray, array, n);
	t1 = std::chrono::high_resolution_clock::now();
	radixSort(array, n, maxValue);
	t2 = std::chrono::high_resolution_clock::now();
	duration = std::chrono::duration_cast<std::chrono::milliseconds>(t2 - t1).count();
	std::cout << "radixSort duration: " << duration << " ms.\n";

	copyArray(randomArray, array, n);
	t1 = std::chrono::high_resolution_clock::now();
	countSort(array, n, maxValue);
	t2 = std::chrono::high_resolution_clock::now();
	duration = std::chrono::duration_cast<std::chrono::milliseconds>(t2 - t1).count();
	std::cout << "countSort duration: " << duration << " ms.\n";

	copyArray(randomArray, array, n);
	t1 = std::chrono::high_resolution_clock::now();
	mergeSort(array, n);
	t2 = std::chrono::high_resolution_clock::now();
	duration = std::chrono::duration_cast<std::chrono::milliseconds>(t2 - t1).count();
	std::cout << "mergeSort duration: " << duration << " ms.\n";

	copyArray(randomArray, array, n);
//	showArray(array, n);
	t1 = std::chrono::high_resolution_clock::now();
	multithreadMergeSort(array, n);
	t2 = std::chrono::high_resolution_clock::now();
	duration = std::chrono::duration_cast<std::chrono::milliseconds>(t2 - t1).count();
	std::cout << "multithreadMergeSort duration: " << duration << " ms.\n";
	std::cout << std::boolalpha << "Is array sorted: " << isArraySorted(array, n) << std::endl;
	std::cout << "Is array checksum right: "
	          << (getArrayChecksum(array, n) == checksum) << std::endl;
//	showArray(array, n);

	delete[] randomArray;
	delete[] array;

	return 0;
}
