#include <iostream>
struct A { int x, y; };

// Структури не можна порівнювати прямо.
// Для цього треба визначити оператор == в якому треба порівняти елементи структури.
bool operator == (const A& a1, const A& a2) { return a1.x == a2.x && a1.y == a2.y; }
// Без даного оператора == виникне помилка компіляції
// no match for ‘operator==’ (operand types are ‘A’ and ‘A’).
// В даний оператор == екземпляри структур передаються по посилані.

// Також допустимо передавати в функцію екземпляр структури по значенню.
// Функція може повернути значення як структуру.
A f(A arg) {
	std::cout << "arg.x: " << arg.x << ", arg.y: " << arg.y << std::endl;
	A a = arg;
	a.x++;
	a.y++;
	return a;
}

int main() {
	A a1 = {1, 2}, a2 = {1, 2}, a3 = {3, 4};
	std::cout << std::boolalpha << (a1 == a3) << std::endl << (a1 == a2) << std::endl;

	std::cout << "a1.x: " << a1.x << ", a1.y: " << a1.y << std::endl;
	std::cout << "a1 address: " << &a1
	     << ", a1.x address: " << &a1.x << ", a1.y address: " << &a1.y << std::endl;

	// Пряме присвоєння значення структури допустиме.
	// В цьому випадку значення структури копіюється побайтово.
	a1 = a3;

	std::cout << "a1.x: " << a1.x << ", a1.y: " << a1.y << std::endl;

	// Адреса a1 не змінилась. Оператор = скопіював саме значення елементів структури.
	std::cout << "a1 address: " << &a1
	     << ", a1.x address: " << &a1.x << ", a1.y address: " << &a1.y << std::endl;

	// Адреса екземпляру структури співпадає з адресою її першого елементу.
	std::cout << "&a1 == &a1.x : "
	     << ( reinterpret_cast<long>(&a1) == reinterpret_cast<long>(&a1.x) ) << std::endl;
	// Не можна прямо порівнювати вказівки різних типів.

	A r = f(a1);
	std::cout << "r.x: " << r.x << ", r.y: " << r.y << std::endl;
	return 0;
}
