#include <iostream>

/*
union по суті це все та ж структура, за винятком того, що всі змінні займають один і
той же обсяг пам'яті. Коли в програмі оголошується union, компілятор виділяє достатньо
пам'яті для найбільшого типу даних в об'єднанні.
Елементи union ніби накладаються на один одного і мають однакову адресу.
*/

union A { long l; int i; double d; };

struct B {
	int i;
	union { long l; double d; }; // Безімена union.
};

int main() {
	std::cout << "sizeof(long): "<< sizeof(long) << std::endl; // 8
	std::cout << "sizeof(int): "<< sizeof(int) << std::endl; // 4
	std::cout << "sizeof(double): "<< sizeof(double) << std::endl; // 1
	std::cout << "sizeof(A): "<< sizeof(A) << std::endl; // 8

	A a = {};
	// Змінна a та її елементи мають однакову адресу.
	std::cout << "  a address: " << &a << std::endl;
	std::cout << "a.l address: " << &a.l << std::endl;
	std::cout << "a.i address: " << &a.i << std::endl;
	std::cout << "a.d address: " << &a.d << std::endl;

	// Одночасно можна використовувати лише один елемент union
	// тому що зміна значення одного елементу приведе до зміни значень інших елементів.
	std::cout << "a.l: " << a.l << std::endl;
	std::cout << "a.i: " << a.i << std::endl;
	std::cout << "a.d: " << a.d << std::endl;
	a.d = 5.9;
	std::cout << "a.l: " << a.l << std::endl;
	std::cout << "a.i: " << a.i << std::endl;
	std::cout << "a.d: " << a.d << std::endl;

	B b = {};
	std::cout << "sizeof(b): "<< sizeof(b) << std::endl; // 16
	b.l = 9;
	std::cout << "b.d: " << b.d << std::endl;

	union { long l = 0; int i; };
	std::cout << "l: " << l << std::endl; // 0
	i = 9;
	std::cout  << "l: " << l << std::endl; // 9
	return 0;
}
