#include <iostream>

// Class which acts as a polymorphic base class
// usually needs either a virtual destructor or virtual functions.

struct A {
	void f1() {
		std::cout << "A::f1 була викликана.\n";

		// Функція A::f2 віртуальна.
		// Тому на цьому місці, якщо створено екземпляр класу B,
		// замість A::f2 буде викликана функція f2 з похідного класу B.
		f2();
		// Якби похідний клас не мав своєї функції f2, то була б викликана функція A::f2.

		// Але якщо створено екземпляр класу A, буде викликана функція A::f2.
	}
	virtual void f2() { std::cout << "A::f2 була викликана.\n"; }
	virtual void f3() { std::cout << "A::f3 була викликана.\n"; }
};

struct B : A {
	void f2() { std::cout << "B::f2 була викликана.\n"; }
	void f3() { std::cout << "B::f3 була викликана.\n"; }
	void f4() { std::cout << "B::f4 була викликана.\n"; }
};

int main() {
	B b;
	b.f1();

	A* p = &b;
	// Вказівку p на базовий клас A було отримано з екземпляру класу B.
	// Функція A::f3 віртуальна.
	// Тому замість функції A::f3 буде викликана функція f3 з похідного класу B.
	p->f3(); // Буде викликана B::f3.

	// Отримати доступ до звичайних елементів похідного класу з вказівки на базовий клас
	// можна оператором dynamic_cast.
	// Оператор dynamic_cast допустимий якщо клас,який треба приести,
	// має хоча б одну віртуальну функцію. Інакше
	// трапиться помилка компіляції "cannot dynamic_cast (source type is not polymorphic)".
	dynamic_cast<B*>(p)->f4(); // Буде викликана B::f4.

	std::cout << "--------------------------\n";
	A a;
	a.f1();
	p = &a;
	p->f3(); // Буде викликана A::f3.
	return 0;
}
