#include <iostream>

// vtable це масив вказівок на віртуальні або перекриті функції.
// Перші 8 байт поліморфного класу зберігають адресу цього масиву.
// vtable має лише поліморфний клас.
// Клас поліморфний якщо має віртуальний деструктор або хоча б одну віртуальну функцію.

// Коли створюється екземпляр базового поліморфного класу,
// в vtable записуються адреси віртуальних функцій.

// Коли створюється екземпляр похідного поліморфного класу,
// в vtable записуються адреси функцій похідного класу,
// які перекрили віртуальні функції базового класу.
// Тому базовий поліморфний клас може викликати перекриті функції з похідного класу.

// Виклик віртуальних або перекритих функцій відбувається неявно через vtable.

struct A {
	void check() {
		// Емуляція виклику віртуальних або перекритих функцій.
		// Даний приклад показано щоб зрозуміти принцип роботи поліморфного класу.
		// У реальній програмі таке не використовуть.
		void (*f)();
		long* vtablePointer = (long*) *((long*)this); // Отримуємо вказівку на vtable.
		f = (void(*)()) vtablePointer[0];
		f(); // Виклик f1.
		f = (void(*)()) vtablePointer[1];
		f(); // Виклик f2.
	}

	virtual void f1() { std::cout << "A::f1 була викликана.\n"; }
	virtual void f2() { std::cout << "A::f2 була викликана.\n"; }
};

struct B : A {
	void f1() { std::cout << "B::f1 була викликана.\n"; }
	void f2() { std::cout << "B::f2 була викликана.\n"; }
};

struct C : A {
	void f1() { std::cout << "C::f1 була викликана.\n"; }
	void f2() { std::cout << "C::f2 була викликана.\n"; }
};

int main() {
	A a; B b; C c;
	a.check();
	b.check();
	A& reference = c;
	reference.check();
	return 0;
}
