#include <iostream>
#include <cstring>
#include <boost/algorithm/string/predicate.hpp>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

const int port = 9000;
const char html_document[] =
		"<!DOCTYPE HTML>\n"
		"<html>\n"
		"<head>\n"
		"<meta charset=\"utf-8\">\n"
		"<title>Check</title>\n"
		"</head>\n"
		"<body>\n"
		"<h1 style='text-align: center'>Hello.</h1>\n"
		"</body>\n"
		"</html>";

void error(const char *msg) {
	perror(msg);
	exit(1);
}

int main() {
	int sockfd = socket(AF_INET, SOCK_STREAM, 0);
	if (sockfd < 0) error("Error opening socket");

	char buffer[256];
	sockaddr_in server_addr = {}, client_addr;
	/*
	cout << "server_addr bytes: ";
	for (const char *p = (char *)&server_addr;
		 p != ((char *)&server_addr) + sizeof(sockaddr_in);
		 p++) cout << (int)*p <<  ' ';
	cout << "\nSize of sockaddr_in: " << sizeof(sockaddr_in) << endl;
	*/
	server_addr.sin_family = AF_INET;
	server_addr.sin_port = htons(port);

	// Automatically will be filled with current host's IP address.
	server_addr.sin_addr.s_addr = INADDR_ANY;

	if (bind(sockfd, (sockaddr *)&server_addr, sizeof(server_addr)) < 0)
		error("Error on binding");
	listen(sockfd, 5);
	std::cout << "The server listens on port " << port << ".\n";

	for (;;) {
		// accept блокує програму дло тих пір поки не зявиться нове підключення.
		socklen_t client_addr_len = sizeof(client_addr);
		int newsockfd = accept(sockfd, (sockaddr *)&client_addr, &client_addr_len);
		if (newsockfd < 0) error("Error on accept");

		std::cout << "Server: got connection from " << inet_ntoa(client_addr.sin_addr)
		          << " port " << ntohs(client_addr.sin_port) << std::endl;

		bzero(buffer, sizeof(buffer));
		int n = read(newsockfd, buffer, sizeof(buffer) - 1);
		if (n < 0) error("Error reading from socket");
		std::cout << "Message:\n" << buffer << std::endl;

		if (boost::starts_with(buffer, "GET / HTTP/1.1")) {
			std::string response("HTTP/1.1 200 OK\r\n"
							"Version: HTTP/1.1\r\n"
							"Content-Type: text/html; charset=utf-8\r\n");
			response += "Content-Length: ";
			response += std::to_string(strlen(html_document));
			response += "\r\n\r\n";
			response += html_document;

			send(newsockfd, response.c_str(), response.length(), 0);
		}
		close(newsockfd);
	}
	close(sockfd);
	return 0;
}
