#include <windows.h>
#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <vector>
#define MAXBLOCK 80

HANDLE hCom; OVERLAPPED oRead = {0}, oWrite = {0};


int ReadCommBlock(LPSTR lpszBlock, int nMaxLength) {
	COMSTAT ComStat; DWORD dwErrorFlags, dwLength, dwError;
	ClearCommError(hCom, &dwErrorFlags, &ComStat);
	dwLength = min((DWORD)nMaxLength, ComStat.cbInQue);
	if (dwLength > 0)	{
		if (!ReadFile(hCom, lpszBlock, dwLength, &dwLength, &oRead)) {
			if (GetLastError() == ERROR_IO_PENDING) {
				// We have to wait for read to complete. 
				// This function will timeout according to the
				// CommTimeOuts.ReadTotalTimeoutConstant variable
				// Every time it times out, check for port errors
				while (GetOverlappedResult(hCom, &oRead,
				                           &dwLength,
				                           TRUE) == FALSE) {
					dwError = GetLastError();
					if (dwError != ERROR_IO_INCOMPLETE) { // error occurred, try to recover
						printf(sz,
						       "Read error. %d.\n",
						        dwError);
						ClearCommError(hCom, &dwErrorFlags, &ComStat);
						break;
					}
				}
			}
		}
	}
	return dwLength;
} 


DWORD WINAPI TerminalThread(LPVOID arg) {
	DWORD dwEvtMask; int nLength, i;
	BYTE abIn[MAXBLOCK + 1];
	
	for (;;) {
		dwEvtMask = 0;
		WaitCommEvent(hCom, &dwEvtMask, 0);
		if ((dwEvtMask & EV_RXCHAR) == EV_RXCHAR) {
			do {
				nLength = ReadCommBlock((LPSTR)abIn, MAXBLOCK);
				if (nLength) {

					std::vector<char> v((char*)abIn,
					                    (char*)abIn + nLength);
					v.push_back('\0');

					i = 0;
					while (i < (v.size() - 1)) {
						switch (v[i]) {
						case '\r':
							if (v[i + 1] != '\n')
								v.insert(v.begin() + i + 1, 
								         '\n');
							i += 2;
							break;
						case '\n':
							v.insert(v.begin() + i,
							         '\r');
							i += 2;
							break;
						default:
							i++; //    
						}
					}

					printf("%s", (char*)v.begin());
				}
			} while (nLength > 0);
		}
	}
	return 0;
}


// FALSE - error.
BOOL InitApp() {
	hCom = CreateFile(
					"COM1",
					GENERIC_READ | GENERIC_WRITE, 0,
					NULL, OPEN_EXISTING,
					FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED,
					NULL);
	if (hCom == INVALID_HANDLE_VALUE) {
		printf("COM1-port open error.\n");
		return FALSE;
	}
	
	BOOL bSuccess;
	DCB dcb;
	bSuccess = GetCommState(hCom, &dcb);
	assert(bSuccess);
	dcb.BaudRate = 38400;
	dcb.ByteSize = 8;
	dcb.Parity = NOPARITY;
	dcb.StopBits = ONESTOPBIT;
	bSuccess = SetCommState(hCom, &dcb);
	assert(bSuccess);

	COMMTIMEOUTS CommTimeouts;
	CommTimeouts.ReadIntervalTimeout = 50;
	CommTimeouts.ReadTotalTimeoutMultiplier = 1;
	CommTimeouts.ReadTotalTimeoutConstant = 50;
	CommTimeouts.WriteTotalTimeoutMultiplier = 1;
	CommTimeouts.WriteTotalTimeoutConstant = 50;
	bSuccess = SetCommTimeouts(hCom, &CommTimeouts);
	assert(bSuccess);

	FlushFileBuffers(hCom);

	oRead.hEvent = CreateEvent(0, TRUE, FALSE, 0);
	assert(oRead.hEvent);

	bSuccess = SetCommMask(hCom, EV_RXCHAR);
	assert(bSuccess);

	DWORD dw;
	CreateThread(NULL,0,TerminalThread,NULL,0,&dw);

	dw = 0;
	char sw[] = "AT&FE1S0=1\r";
	oWrite.hEvent = CreateEvent(0, TRUE, FALSE, 0);
	assert(oWrite.hEvent);   
	WriteFile(hCom, sw, strlen(sw), &dw, &oWrite);
	
	return TRUE;
}


int main() {
	if (InitApp() == FALSE) return 0;
	int i; DWORD dw; char c;
	printf("F10 - exit.\n");
	while ( (i = getch()) != 68 ) {
		// printf("Typed key: %d.\n\n", c);
		c = i; dw = 0;
		WriteFile(hCom, &c, 1, &dw, &oWrite);
	}
	CloseHandle(hCom);
	return 0;
}
