#include <GL/glut.h>

const GLfloat light_ambient[]  = { 0.0f, 0.0f, 0.0f, 1.0f };
const GLfloat light_diffuse[]  = { 1.0f, 1.0f, 1.0f, 1.0f };
const GLfloat light_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
const GLfloat light_position[] = { 2.0f, 5.0f, 5.0f, 0.0f };
const GLfloat mat_ambient[]    = { 0.7f, 0.7f, 0.7f, 1.0f };
const GLfloat mat_diffuse[]    = { 0.8f, 0.8f, 0.8f, 1.0f };
const GLfloat mat_specular[]   = { 1.0f, 1.0f, 1.0f, 1.0f };
const GLfloat high_shininess[] = { 100.0f };

void renderScene() {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();
	GLUquadric* quad = gluNewQuadric();
	glColor3d(0, 1, 0);

	glTranslated(-3.0, 1.0, -10); // glTranslated переміщає точку відліку.
	gluSphere(quad, 0.3, 100, 20); // Сфера.

	glTranslated(0.0, -1.0, 0.0);
	glutSolidSphere(0.3, 100, 20); // Інший варіант сфери.

	glLoadIdentity(); // glLoadIdentity скидає точку відліку.
	glTranslated(-2.0, 1.0, -10);

	// Повертаємо наступну фігуру на 45 градусів навколо x та y.
	glRotated(45, 1, 1, 0);

	gluCylinder(quad, 0.3, 0.3, 1, 15, 15); // Цілиндр.

	glLoadIdentity();
	glTranslated(-0.8, 1, -10);
	gluDisk(quad, 0.15, 0.3, 90, 10); // Диск.

	glTranslated(1, 0, 0);
	glRotated(45, 1, 1, 0);
	glutSolidCube(0.6); // Куб.

	glLoadIdentity();
	glTranslated(1.2, 1, -10);
	glRotated(45, 1, 1, 0);
	glutSolidCone(0.3, 0.6, 15, 15); // Конус.

	glLoadIdentity();
	glTranslated(2.2, 1, -10);
	glRotated(45, 1, 1, 0);
	glutSolidTorus(0.15, 0.3, 90, 10); // Торус.

	gluDeleteQuadric(quad);
	glutSwapBuffers();
}

void windowReshapeFunc(GLint newWidth, GLint newHeight) {
	glViewport(0, 0, newWidth, newHeight);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	GLfloat x = GLfloat(newWidth) / newHeight;
	glFrustum(-x, x, -1.0, 1.0, 4.0, 15.0);
	glMatrixMode(GL_MODELVIEW);
}

int main(int argc, char** argv) {
	glutInit(&argc, argv);

	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition(80, 10);
	glutInitWindowSize(1200, 700);
	glutCreateWindow("3d primitives");

	glClearColor(0.0, 0.0, 0.0, 0.0);

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);
	glEnable(GL_NORMALIZE);
	glEnable(GL_COLOR_MATERIAL);

	glLightfv(GL_LIGHT0, GL_AMBIENT,  light_ambient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE,  light_diffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);

	glMaterialfv(GL_FRONT, GL_AMBIENT,   mat_ambient);
	glMaterialfv(GL_FRONT, GL_DIFFUSE,   mat_diffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR,  mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, high_shininess);

	// Реєстрація зворотних викликів.
	glutDisplayFunc(renderScene);
	glutReshapeFunc(windowReshapeFunc);

	glutMainLoop();
	return 0;
}
